import django
from fluent_faq.admin.base import FaqBaseModelAdmin
from fluent_faq.models import FaqQuestion

if django.VERSION >= (1, 8):
    # get_all_field_names() was deprecated
    _model_fields = [f.name for f in FaqQuestion._meta.get_fields()]
else:
    # This also returns any _id fields, but that's not an issue for us here.
    _model_fields = FaqQuestion._meta.get_all_field_names()


class FaqQuestionAdmin(FaqBaseModelAdmin):
    """
    Admin interface for the FAQ Question model.
    """
    list_display = ('title', 'language_column', 'category', 'modification_date', 'actions_column')
    list_filter = ('category',)

    FIELDSET_GENERAL = (None, {
        'fields': ('title', 'slug', 'contents',),
    })

    fieldsets = (
        FIELDSET_GENERAL,
        FaqBaseModelAdmin.FIELDSET_PUBLICATION,
        FaqBaseModelAdmin.FIELDSET_SEO,
    )

    def get_queryset(self, request):
        return super(FaqQuestionAdmin, self).get_queryset(request).select_related('category')

    if django.VERSION < (1, 6):
        queryset = get_queryset


# Add all fields
for _f in ('category', 'tags'):
    if _f in _model_fields:
        FaqQuestionAdmin.FIELDSET_GENERAL[1]['fields'] += (_f,)



from django.conf import settings
from django.template import Library
from tag_parser import template_tag
from tag_parser.basetags import BaseAssignmentOrOutputNode

register = Library()

FaqPage = None
HAS_APP_URLS = 'fluent_pages' in settings.INSTALLED_APPS
if HAS_APP_URLS:
    # HACK: accessing FaqPage directly. Apps are not completely separated this way.
    # Should have some kind of registry and filter system (like middleware) instead.
    from fluent_faq.pagetypes.faqpage.models import FaqPage


@template_tag(register, 'get_faq_url')
class GetFaqUrl(BaseAssignmentOrOutputNode):
    """
    Get the URL of a FAQ question.

    When using django-fluent-pages, this takes the current ``page`` variable into account.
    It makes sure the FAQ question is relative to the current page.

    When django-fluent-pages is not used, using this is identical to calling ``faqquestion.get_absolute_url()``.
    """
    min_args = 1
    max_args = 1
    takes_context = True

    def get_value(self, context, *tag_args, **tag_kwargs):
        entry = tag_args[0]

        if HAS_APP_URLS:
            # If the application supports mounting a BlogPage in the page tree,
            # that can be used as relative start point of the entry.
            page = context.get('page')
            request = context.get('request')
            if page is None and request is not None:
                # HACK: access private django-fluent-pages var
                page = getattr(request, '_current_fluent_page', None)

            if page is not None and isinstance(page, FaqPage):
                return page.get_object_url(entry)

        return entry.get_absolute_url()



from fluent_utils.softdeps.fluent_pages import mixed_reverse


def faq_reverse(viewname, args=None, kwargs=None, current_app='fluent_faq', **page_kwargs):
    """
    Reverse a URL to the FAQ, taking various configuration options into account.

    This is a compatibility function to allow django-fluent-faq to operate stand-alone.
    Either the app can be hooked in the URLconf directly, or it can be added as a pagetype of *django-fluent-pages*.
    """
    return mixed_reverse(viewname, args=args, kwargs=kwargs, current_app=current_app, **page_kwargs)


__all__ = (
    'faq_reverse',
)



from django.utils.translation import ugettext_lazy as _
from fluent_faq.models import FaqQuestion
from fluent_pages.models import HtmlPage


class FaqPage(HtmlPage):
    """
    The page root for FAQ questions.
    """

    class Meta:
        verbose_name = _("FAQ module")
        verbose_name_plural = _("FAQ modules")

    @property
    def questions(self):
        """
        Return the entries that are published under this node.
        """
        return FaqQuestion.objects.all()

    def get_object_url(self, object):
        """
        Return the URL of a FaqQuestion, relative to this page.
        """
        return self.get_absolute_url() + object.get_relative_url()



from django.conf.urls import url
from .views import FaqQuestionList, FaqCategoryDetail, FaqQuestionDetail

urlpatterns = [
    url(r'^$', FaqQuestionList.as_view(), name='faqquestion_index'),
    url(r'^(?P<slug>[^/]+)/$', FaqCategoryDetail.as_view(), name='faqcategory_detail'),
    url(r'^(?P<cat_slug>[^/]+)/(?P<slug>[^/]+)/$', FaqQuestionDetail.as_view(), name='faqquestion_detail'),
]



from django.conf import settings
from django.contrib.admin.widgets import AdminTextInputWidget, AdminTextareaWidget
from django.core.exceptions import ValidationError, ImproperlyConfigured
from django.core.urlresolvers import NoReverseMatch
from django.utils.html import conditional_escape
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext, ugettext_lazy as _
from fluent_faq import appsettings
from fluent_contents.admin import PlaceholderFieldAdmin
from fluent_utils.softdeps.fluent_pages import mixed_reverse
from fluent_utils.dry.admin import MultiSiteAdminMixin
from parler.admin import TranslatableAdmin
from parler.forms import TranslatableModelForm
from parler.models import TranslationDoesNotExist


class FaqBaseModelForm(TranslatableModelForm):
    """
    Base form for FAQ questions.
    """

    def clean_slug(self):
        """
        Test whether the slug is unique within the language domain.
        """
        slug = self.cleaned_data['slug']
        dup_qs = self._meta.model.objects.translated(self.language_code, slug=slug)
        if self.instance and self.instance.pk:
            dup_qs = dup_qs.exclude(pk=self.instance.pk)

        if dup_qs.exists():
            raise ValidationError(_("The slug is not unique"))

        return slug


class FaqBaseModelAdmin(MultiSiteAdminMixin, TranslatableAdmin, PlaceholderFieldAdmin):
    """
    Base admin for FAQ questions
    """
    filter_site = appsettings.FLUENT_FAQ_FILTER_SITE_ID
    list_display = ('title', 'language_column', 'modification_date', 'actions_column')
    form = FaqBaseModelForm
    search_fields = ('translations__slug', 'translations__title')

    # Using fieldnames here works because formfield_for_dbfield() is overwritten.
    formfield_overrides = {
        'meta_keywords': {
            'widget': AdminTextInputWidget(attrs={'class': 'vLargeTextField'})
        },
        'meta_description': {
            'widget': AdminTextareaWidget(attrs={'rows': 3})
        },
    }

    FIELDSET_GENERAL = (None, {
        'fields': ('title', 'slug',),
    })
    FIELDSET_PUBLICATION = (_('Publication settings'), {
        'fields': ('order',),
        #'classes': ('collapse',),
    })
    FIELDSET_SEO = (_('SEO settings'), {
        'fields': ('meta_keywords', 'meta_description'),
        'classes': ('collapse',),
    })

    fieldsets = (
        FIELDSET_GENERAL,
        FIELDSET_PUBLICATION,
        FIELDSET_SEO,
    )

    class Media:
        css = {
            'all': ('fluent_faq/admin/admin.css',)
        }

    def get_prepopulated_fields(self, request, obj=None):
        # Needed for django-parler
        return {'slug': ('title',), }

    def save_model(self, request, obj, form, change):
        # Automatically store the user in the author field.
        if not change:
            obj.author = request.user
        obj.save()

    def formfield_for_dbfield(self, db_field, **kwargs):
        """
        Allow formfield_overrides to contain field names too.
        """
        overrides = self.formfield_overrides.get(db_field.name)
        if overrides:
            kwargs.update(overrides)

        return super(FaqBaseModelAdmin, self).formfield_for_dbfield(db_field, **kwargs)

    def render_change_form(self, request, context, add=False, change=False, form_url='', obj=None):
        # When the page is accessed via a pagetype, warn that the node can't be previewed yet.
        context['preview_error'] = ''
        if 'fluent_pages' in settings.INSTALLED_APPS:
            from fluent_pages.urlresolvers import PageTypeNotMounted, MultipleReverseMatch
            try:
                self._reverse_faqpage_index(request, obj)
            except PageTypeNotMounted:
                from fluent_faq.pagetypes.faqpage.models import FaqPage
                context['preview_error'] = ugettext("The {object_name} can't be previewed yet, a '{page_type_name}' page needs to be created first.").format(object_name=self.model._meta.verbose_name, page_type_name=FaqPage._meta.verbose_name)
            except MultipleReverseMatch:
                # When 'faqquestion_index is ambiguous (because there are multiple FAQ nodes in the fluent-pages tree),
                # the edit page will automatically pick an option.
                pass
            except NoReverseMatch:
                # Since forgetting the pagetype app is easy, give off a warning to help developers
                # find their way with these apps.
                raise ImproperlyConfigured(
                    "To use django-fluent-faq, either include('fluent_faq.urls') in the URLConf, "
                    "or add the 'fluent_faq.pagetypes.faqpage' app to the INSTALLED_APPS."
                )

        return super(FaqBaseModelAdmin, self).render_change_form(request, context, add, change, form_url, obj)

    def _reverse_faqpage_index(self, request, obj=None):
        # Internal method with "protected access" to handle translation differences.
        # This is only called when 'fluent_pages' is in the INSTALLED_APPS.
        return mixed_reverse('faqquestion_index')

    @classmethod
    def get_actions_column(cls, faqquestion):
        return mark_safe(u' '.join(conditional_escape(a) for a in cls._actions_column_icons(faqquestion)))

    @classmethod
    def _actions_column_icons(cls, object):
        actions = []
        if cls.can_preview_object(object):
            try:
                url = object.get_absolute_url()
            except (NoReverseMatch, TranslationDoesNotExist):
                # A FaqQuestion is already added, but the URL can no longer be resolved.
                # This can either mean that urls.py is missing a 'fluent_faq.urls' (unlikely),
                # or that this is a PageTypeNotMounted exception because the "FAQ page" node was removed.
                # In the second case, the edit page should still be reachable, and the "view on site" link will give an alert.
                pass
            else:
                actions.append(mark_safe(
                    u'<a href="{url}" title="{title}" target="_blank"><img src="{static}fluent_faq/img/admin/world.gif" width="16" height="16" alt="{title}" /></a>'.format(
                        url=url, title=_('View on site'), static=settings.STATIC_URL)
                ))
        return actions

    @classmethod
    def can_preview_object(cls, object):
        """ Override whether the node can be previewed. """
        #return hasattr(faqquestion, 'get_absolute_url') and faqquestion.is_published
        return True

    def actions_column(self, faqquestion):
        return self.get_actions_column(faqquestion)

    actions_column.allow_tags = True
    actions_column.short_description = _('Actions')



from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from parler import appsettings as parler_appsettings
from parler.utils import normalize_language_code, is_supported_django_language

# Advanced settings
FLUENT_FAQ_FILTER_SITE_ID = getattr(settings, 'FLUENT_FAQ_FILTER_SITE_ID', True)
FLUENT_FAQ_BASE_TEMPLATE = getattr(settings, "FLUENT_FAQ_BASE_TEMPLATE", 'fluent_faq/base.html')

# Performance settings
FLUENT_FAQ_PREFETCH_TRANSLATIONS = getattr(settings, 'FLUENT_FAQ_PREFETCH_TRANSLATIONS', False)

# Note: the default language setting is used during the migrations
# Allow this module to have other settings, but default to the shared settings
FLUENT_DEFAULT_LANGUAGE_CODE = getattr(settings, 'FLUENT_DEFAULT_LANGUAGE_CODE', parler_appsettings.PARLER_DEFAULT_LANGUAGE_CODE)
FLUENT_FAQ_DEFAULT_LANGUAGE_CODE = getattr(settings, 'FLUENT_FAQ_DEFAULT_LANGUAGE_CODE', FLUENT_DEFAULT_LANGUAGE_CODE)
FLUENT_FAQ_LANGUAGES = getattr(settings, 'FLUENT_FAQ_LANGUAGES', parler_appsettings.PARLER_LANGUAGES)


# Clean settings
FLUENT_FAQ_DEFAULT_LANGUAGE_CODE = normalize_language_code(FLUENT_FAQ_DEFAULT_LANGUAGE_CODE)

if not is_supported_django_language(FLUENT_FAQ_DEFAULT_LANGUAGE_CODE):
    raise ImproperlyConfigured("FLUENT_FAQ_DEFAULT_LANGUAGE_CODE '{0}' does not exist in LANGUAGES".format(FLUENT_FAQ_DEFAULT_LANGUAGE_CODE))

FLUENT_FAQ_LANGUAGES = parler_appsettings.add_default_language_settings(
    FLUENT_FAQ_LANGUAGES, 'FLUENT_FAQ_LANGUAGES',
    code=FLUENT_FAQ_DEFAULT_LANGUAGE_CODE,
    fallback=FLUENT_FAQ_DEFAULT_LANGUAGE_CODE,
    hide_untranslated=False,
)



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'FaqCategoryTranslation'
        db.create_table(u'fluent_faq_faqcategory_translation', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('language_code', self.gf('django.db.models.fields.CharField')(max_length=15, db_index=True)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=200)),
            ('slug', self.gf('django.db.models.fields.SlugField')(max_length=50)),
            (u'master', self.gf('django.db.models.fields.related.ForeignKey')(related_name='translations', null=True, to=orm['fluent_faq.FaqCategory'])),
        ))
        db.send_create_signal(u'fluent_faq', ['FaqCategoryTranslation'])

        # Adding unique constraint on 'FaqCategoryTranslation', fields ['language_code', u'master']
        db.create_unique(u'fluent_faq_faqcategory_translation', ['language_code', u'master_id'])

        # Adding model 'FaqCategory'
        db.create_table(u'fluent_faq_faqcategory', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('meta_keywords', self.gf('django.db.models.fields.CharField')(default='', max_length=255, blank=True)),
            ('meta_description', self.gf('django.db.models.fields.CharField')(default='', max_length=255, blank=True)),
            ('author', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.User'])),
            ('creation_date', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('modification_date', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, blank=True)),
            ('parent_site', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['sites.Site'])),
            ('order', self.gf('django.db.models.fields.PositiveIntegerField')(db_index=True, null=True, blank=True)),
        ))
        db.send_create_signal(u'fluent_faq', ['FaqCategory'])

        # Adding model 'FaqQuestionTranslation'
        db.create_table(u'fluent_faq_faqquestion_translation', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('language_code', self.gf('django.db.models.fields.CharField')(max_length=15, db_index=True)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=200)),
            ('slug', self.gf('django.db.models.fields.SlugField')(max_length=50)),
            (u'master', self.gf('django.db.models.fields.related.ForeignKey')(related_name='translations', null=True, to=orm['fluent_faq.FaqQuestion'])),
        ))
        db.send_create_signal(u'fluent_faq', ['FaqQuestionTranslation'])

        # Adding unique constraint on 'FaqQuestionTranslation', fields ['language_code', u'master']
        db.create_unique(u'fluent_faq_faqquestion_translation', ['language_code', u'master_id'])

        # Adding model 'FaqQuestion'
        db.create_table(u'fluent_faq_faqquestion', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('meta_keywords', self.gf('django.db.models.fields.CharField')(default='', max_length=255, blank=True)),
            ('meta_description', self.gf('django.db.models.fields.CharField')(default='', max_length=255, blank=True)),
            ('author', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.User'])),
            ('creation_date', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('modification_date', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, blank=True)),
            ('parent_site', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['sites.Site'])),
            ('order', self.gf('django.db.models.fields.PositiveIntegerField')(db_index=True, null=True, blank=True)),
            ('category', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['fluent_faq.FaqCategory'])),
        ))
        db.send_create_signal(u'fluent_faq', ['FaqQuestion'])


    def backwards(self, orm):
        # Removing unique constraint on 'FaqQuestionTranslation', fields ['language_code', u'master']
        db.delete_unique(u'fluent_faq_faqquestion_translation', ['language_code', u'master_id'])

        # Removing unique constraint on 'FaqCategoryTranslation', fields ['language_code', u'master']
        db.delete_unique(u'fluent_faq_faqcategory_translation', ['language_code', u'master_id'])

        # Deleting model 'FaqCategoryTranslation'
        db.delete_table(u'fluent_faq_faqcategory_translation')

        # Deleting model 'FaqCategory'
        db.delete_table(u'fluent_faq_faqcategory')

        # Deleting model 'FaqQuestionTranslation'
        db.delete_table(u'fluent_faq_faqquestion_translation')

        # Deleting model 'FaqQuestion'
        db.delete_table(u'fluent_faq_faqquestion')


    models = {
        u'auth.group': {
            'Meta': {'object_name': 'Group'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        u'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        u'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'fluent_contents.contentitem': {
            'Meta': {'ordering': "('placeholder', 'sort_order')", 'object_name': 'ContentItem'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language_code': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '15', 'db_index': 'True'}),
            'parent_id': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'parent_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            'placeholder': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'contentitems'", 'null': 'True', 'on_delete': 'models.SET_NULL', 'to': "orm['fluent_contents.Placeholder']"}),
            'polymorphic_ctype': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'polymorphic_fluent_contents.contentitem_set'", 'null': 'True', 'to': u"orm['contenttypes.ContentType']"}),
            'sort_order': ('django.db.models.fields.IntegerField', [], {'default': '1', 'db_index': 'True'})
        },
        'fluent_contents.placeholder': {
            'Meta': {'unique_together': "(('parent_type', 'parent_id', 'slot'),)", 'object_name': 'Placeholder'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'parent_id': ('django.db.models.fields.IntegerField', [], {'null': 'True'}),
            'parent_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']", 'null': 'True', 'blank': 'True'}),
            'role': ('django.db.models.fields.CharField', [], {'default': "'m'", 'max_length': '1'}),
            'slot': ('django.db.models.fields.SlugField', [], {'max_length': '50'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'})
        },
        u'fluent_faq.faqcategory': {
            'Meta': {'ordering': "('order', 'creation_date')", 'object_name': 'FaqCategory'},
            'author': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['auth.User']"}),
            'creation_date': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'meta_description': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '255', 'blank': 'True'}),
            'meta_keywords': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '255', 'blank': 'True'}),
            'modification_date': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'order': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True', 'null': 'True', 'blank': 'True'}),
            'parent_site': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['sites.Site']"})
        },
        u'fluent_faq.faqcategorytranslation': {
            'Meta': {'unique_together': "[(u'language_code', u'master')]", 'object_name': 'FaqCategoryTranslation', 'db_table': "u'fluent_faq_faqcategory_translation'"},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language_code': ('django.db.models.fields.CharField', [], {'max_length': '15', 'db_index': 'True'}),
            u'master': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'translations'", 'null': 'True', 'to': u"orm['fluent_faq.FaqCategory']"}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '50'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '200'})
        },
        u'fluent_faq.faqquestion': {
            'Meta': {'ordering': "('order', 'creation_date')", 'object_name': 'FaqQuestion'},
            'author': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['auth.User']"}),
            'category': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['fluent_faq.FaqCategory']"}),
            'creation_date': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'meta_description': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '255', 'blank': 'True'}),
            'meta_keywords': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '255', 'blank': 'True'}),
            'modification_date': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'order': ('django.db.models.fields.PositiveIntegerField', [], {'db_index': 'True', 'null': 'True', 'blank': 'True'}),
            'parent_site': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['sites.Site']"})
        },
        u'fluent_faq.faqquestiontranslation': {
            'Meta': {'unique_together': "[(u'language_code', u'master')]", 'object_name': 'FaqQuestionTranslation', 'db_table': "u'fluent_faq_faqquestion_translation'"},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language_code': ('django.db.models.fields.CharField', [], {'max_length': '15', 'db_index': 'True'}),
            u'master': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'translations'", 'null': 'True', 'to': u"orm['fluent_faq.FaqQuestion']"}),
            'slug': ('django.db.models.fields.SlugField', [], {'max_length': '50'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '200'})
        },
        u'sites.site': {
            'Meta': {'ordering': "('domain',)", 'object_name': 'Site', 'db_table': "'django_site'"},
            'domain': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        u'taggit.tag': {
            'Meta': {'object_name': 'Tag'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '100'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '100'})
        },
        u'taggit.taggeditem': {
            'Meta': {'object_name': 'TaggedItem'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "u'taggit_taggeditem_tagged_items'", 'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'object_id': ('django.db.models.fields.IntegerField', [], {'db_index': 'True'}),
            'tag': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "u'taggit_taggeditem_items'", 'to': u"orm['taggit.Tag']"})
        }
    }

    complete_apps = ['fluent_faq']


from fluent_pages.adminui import HtmlPageAdmin
from fluent_pages.extensions import page_type_pool, PageTypePlugin
from .models import FaqPage


@page_type_pool.register
class FaqPagePlugin(PageTypePlugin):
    """
    Plugin binding the FaqPage model as pagetype.
    """
    model = FaqPage
    model_admin = HtmlPageAdmin
    urls = 'fluent_faq.urls'



from django.contrib.sitemaps import Sitemap
from django.core.urlresolvers import NoReverseMatch
from fluent_faq.models import FaqCategory, FaqQuestion
from fluent_faq.urlresolvers import faq_reverse


def _url_patterns_installed():
    # This module can use normal Django urls.py URLs, or mount the "FaqPage" in the page tree.
    # Check whether the URLs are installed, so the `sitemap.xml` can be generated nevertheless.
    # This issue will pop up elsewhere too, so there is no need to raise an error here.
    try:
        faq_reverse('faqcategory_detail', kwargs={'slug': 'category'}, ignore_multiple=True)
    except NoReverseMatch:
        return False
    else:
        return True


class FaqQuestionSitemap(Sitemap):
    """
    Sitemap for FAQ questions
    """

    def items(self):
        if not _url_patterns_installed():
            return []
        return FaqQuestion.objects.published().select_related('category').active_translations()

    def lastmod(self, question):
        """Return the last modification of the object."""
        return question.modification_date

    def location(self, question):
        """Return url of an question."""
        return faq_reverse('faqquestion_detail', kwargs={'cat_slug': question.category.slug, 'slug': question.slug}, ignore_multiple=True)


class FaqCategorySitemap(Sitemap):
    """
    Sitemap for FAQ categories.
    """

    def items(self):
        if not _url_patterns_installed():
            return []
        return FaqCategory.objects.published().active_translations()

    def lastmod(self, category):
        """Return the last modification of the object."""
        return category.modification_date

    def location(self, category):
        """Return url of an category."""
        return faq_reverse('faqcategory_detail', kwargs={'slug': category.slug}, ignore_multiple=True)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings
import fluent_faq.models


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='FaqCategory',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('meta_keywords', models.CharField(default=b'', help_text='When this field is not filled in, the the tags will be used.', max_length=255, verbose_name='keywords', blank=True)),
                ('meta_description', models.CharField(default=b'', help_text='When this field is not filled in, the contents or intro text will be used.', max_length=255, verbose_name='description', blank=True)),
                ('creation_date', models.DateTimeField(auto_now_add=True, verbose_name='creation date')),
                ('modification_date', models.DateTimeField(auto_now=True, verbose_name='last modification')),
                ('order', models.PositiveIntegerField(db_index=True, null=True, blank=True)),
                ('author', models.ForeignKey(editable=False, to=settings.AUTH_USER_MODEL, verbose_name='author')),
                ('parent_site', models.ForeignKey(default=fluent_faq.models._get_current_site, editable=False, to='sites.Site')),
            ],
            options={
                'ordering': ('order', 'creation_date'),
                'verbose_name': 'FAQ Category',
                'verbose_name_plural': 'FAQ Categories',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='FaqCategoryTranslation',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('language_code', models.CharField(max_length=15, verbose_name='Language', db_index=True)),
                ('title', models.CharField(max_length=200, verbose_name='title')),
                ('slug', models.SlugField(verbose_name='slug')),
                ('master', models.ForeignKey(related_name='translations', editable=False, to='fluent_faq.FaqCategory', null=True)),
            ],
            options={
                'managed': True,
                'db_table': 'fluent_faq_faqcategory_translation',
                'db_tablespace': '',
                'default_permissions': (),
                'verbose_name': 'FAQ Category Translation',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='FaqQuestion',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('meta_keywords', models.CharField(default=b'', help_text='When this field is not filled in, the the tags will be used.', max_length=255, verbose_name='keywords', blank=True)),
                ('meta_description', models.CharField(default=b'', help_text='When this field is not filled in, the contents or intro text will be used.', max_length=255, verbose_name='description', blank=True)),
                ('creation_date', models.DateTimeField(auto_now_add=True, verbose_name='creation date')),
                ('modification_date', models.DateTimeField(auto_now=True, verbose_name='last modification')),
                ('order', models.PositiveIntegerField(db_index=True, null=True, blank=True)),
                ('author', models.ForeignKey(editable=False, to=settings.AUTH_USER_MODEL, verbose_name='author')),
                ('category', models.ForeignKey(related_name='questions', verbose_name='Category', to='fluent_faq.FaqCategory')),
                ('parent_site', models.ForeignKey(default=fluent_faq.models._get_current_site, editable=False, to='sites.Site')),
            ],
            options={
                'ordering': ('order', 'creation_date'),
                'verbose_name': 'FAQ Question',
                'verbose_name_plural': 'FAQ Questions',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='FaqQuestionTranslation',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('language_code', models.CharField(max_length=15, verbose_name='Language', db_index=True)),
                ('title', models.CharField(max_length=200, verbose_name='title')),
                ('slug', models.SlugField(verbose_name='slug')),
                ('master', models.ForeignKey(related_name='translations', editable=False, to='fluent_faq.FaqQuestion', null=True)),
            ],
            options={
                'managed': True,
                'db_table': 'fluent_faq_faqquestion_translation',
                'db_tablespace': '',
                'default_permissions': (),
                'verbose_name': 'FAQ Question Translation',
            },
            bases=(models.Model,),
        ),
        migrations.AlterUniqueTogether(
            name='faqquestiontranslation',
            unique_together=set([('language_code', 'master')]),
        ),
        migrations.AlterUniqueTogether(
            name='faqcategorytranslation',
            unique_together=set([('language_code', 'master')]),
        ),
    ]



from django.conf import settings
from django.contrib.sites.models import Site
from django.db import models
from django.utils.six import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _
from parler.fields import TranslatedField
from parler.models import TranslatableModel, TranslatedFields
from parler.utils.context import switch_language
from fluent_contents.models import PlaceholderField, ContentItemRelation
from fluent_faq.urlresolvers import faq_reverse
from fluent_faq.managers import FaqQuestionManager, FaqCategoryManager
from fluent_utils.softdeps.taggit import TagsMixin


def _get_current_site():
    return Site.objects.get_current().pk


class FaqBaseModel(TranslatableModel):
    """
    Shared functionality for published content.
    """
    # SEO
    meta_keywords = models.CharField(_('keywords'), max_length=255, blank=True, default='', help_text=_("When this field is not filled in, the the tags will be used."))
    meta_description = models.CharField(_('description'), max_length=255, blank=True, default='', help_text=_("When this field is not filled in, the contents or intro text will be used."))

    # Metadata
    author = models.ForeignKey(settings.AUTH_USER_MODEL, verbose_name=_('author'), editable=False)
    creation_date = models.DateTimeField(_('creation date'), editable=False, auto_now_add=True)
    modification_date = models.DateTimeField(_('last modification'), editable=False, auto_now=True)

    parent_site = models.ForeignKey(Site, editable=False, default=_get_current_site)

    class Meta:
        abstract = True

    def get_absolute_url(self):
        return self.default_url

    @property
    def default_url(self):
        """
        The internal implementation of :func:`get_absolute_url`.
        This function can be used when overriding :func:`get_absolute_url` in the settings.
        For example::

            ABSOLUTE_URL_OVERRIDES = {
                'fluent_faq.FaqQuestion': lambda o: "http://example.com" + o.default_url
            }
        """
        with switch_language(self):
            root = faq_reverse('faqquestion_index', ignore_multiple=True, language_code=self.get_current_language())
            return root + self.get_relative_url()

    @property
    def url(self):
        """
        The URL of the entry, provided for template code.
        """
        return self.get_absolute_url()

    def get_relative_url(self):
        raise NotImplementedError("get_relative_url")


@python_2_unicode_compatible
class FaqCategory(FaqBaseModel):
    """
    Topic of the FAQ
    """
    # Be compatible with django-orderable table layout,
    # unfortunately, there isn't a good canonical version of it yet.
    order = models.PositiveIntegerField(db_index=True, blank=True, null=True)

    title = TranslatedField(any_language=True)
    translations = TranslatedFields(
        title = models.CharField(_("title"), max_length=200),
        slug = models.SlugField(_("slug")),
    )

    objects = FaqCategoryManager()

    class Meta:
        verbose_name = _("FAQ Category")
        verbose_name_plural = _("FAQ Categories")
        ordering = ('order', 'creation_date')

    def __str__(self):
        # self.title is configured with any_language=True, so always returns a value.
        return self.title

    def get_relative_url(self):
        return u'{0}/'.format(self.slug)

    @property
    def faq_questions(self):
        """
        Fetch the active FAQ questions in this category.
        """
        return self.questions.active_translations()


@python_2_unicode_compatible
class FaqQuestion(TagsMixin, FaqBaseModel):
    """
    Category in the FAQ.
    """
    # This is a separate model instead of using django-categories because:
    # - content needs to be placed on the category.
    # - the title and slug can be translated!

    # Be compatible with django-orderable table layout,
    # unfortunately, there isn't a good canonical version of it yet.
    order = models.PositiveIntegerField(db_index=True, blank=True, null=True)

    title = TranslatedField(any_language=True)
    translations = TranslatedFields(
        title = models.CharField(_("title"), max_length=200),
        slug = models.SlugField(_("slug")),
    )
    contents = PlaceholderField("faq_answer", verbose_name=_("answer"))
    contentitem_set = ContentItemRelation()  # this makes sure the admin can find all deleted objects too.

    # Organisation
    category = models.ForeignKey(FaqCategory, verbose_name=_("Category"), related_name='questions')

    objects = FaqQuestionManager()

    class Meta:
        verbose_name = _("FAQ Question")
        verbose_name_plural = _("FAQ Questions")
        ordering = ('order', 'creation_date')

    def __str__(self):
        # self.title is configured with any_language=True, so always returns a value.
        return self.title

    def get_relative_url(self):
        """
        Return the link path from the archive page.
        """
        # Return the link style, using the permalink style setting.
        return u'{0}{1}/'.format(self.category.get_relative_url(), self.slug)

    def similar_objects(self, num=None, **filters):
        """
        Find similar objects using related tags.
        """
        #TODO: filter appsettings.FLUENT_FAQ_FILTER_SITE_ID:
        #    filters.setdefault('parent_site', self.parent_site_id)

        # FIXME: Using super() doesn't work, calling directly.
        return TagsMixin.similar_objects(self, num=num, **filters)


def _register_anyurlfield_type():
    try:
        from any_urlfield.models import AnyUrlField
        from any_urlfield.forms.widgets import SimpleRawIdWidget
    except ImportError:
        pass
    else:
        AnyUrlField.register_model(FaqQuestion, widget=SimpleRawIdWidget(FaqQuestion))

if 'any_urlfield' in settings.INSTALLED_APPS:
    _register_anyurlfield_type()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('fluent_pages', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='FaqPage',
            fields=[
                ('urlnode_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='fluent_pages.UrlNode')),
            ],
            options={
                'db_table': 'pagetype_faqpage_faqpage',
                'verbose_name': 'FAQ module',
                'verbose_name_plural': 'FAQ modules',
            },
            bases=('fluent_pages.htmlpage',),
        ),
    ]



from django.http import HttpResponsePermanentRedirect
from django.views.generic import DetailView, ListView
from fluent_faq import appsettings
from fluent_utils.softdeps.fluent_pages import CurrentPageMixin
from fluent_faq.models import FaqCategory, FaqQuestion
from parler.views import TranslatableSlugMixin


class BaseFaqMixin(CurrentPageMixin):
    context_object_name = None
    prefetch_translations = False

    def get_context_data(self, **kwargs):
        context = super(BaseFaqMixin, self).get_context_data(**kwargs)
        context['FLUENT_FAQ_BASE_TEMPLATE'] = appsettings.FLUENT_FAQ_BASE_TEMPLATE
        return context


class FaqQuestionList(BaseFaqMixin, ListView):
    """
    List view for FAQ questions.
    """
    model = FaqQuestion
    view_url_name = 'faqquestion_index'
    prefetch_translations = False

    def get_queryset(self):
        qs = super(FaqQuestionList, self).get_queryset().select_related('category').active_translations()
        if self.prefetch_translations:
            qs = qs.prefetch_related('translations')
        return qs

    def get_context_data(self, **kwargs):
        context = super(FaqQuestionList, self).get_context_data(**kwargs)
        qs = context['object_list']

        groups = {}
        for obj in qs:
            groups.setdefault(obj.category, []).append(obj)

        categories = []
        for category in sorted(groups.keys(), key=lambda c: (c.order, c.title)):
            categories.append(
                (category, groups[category])
            )

        context['categories'] = categories
        return context

    def get_template_names(self):
        names = super(FaqQuestionList, self).get_template_names()  # faqquestion_list.html
        names.insert(0, "fluent_faq/index.html")
        return names


class BaseFaqDetailView(BaseFaqMixin, TranslatableSlugMixin, DetailView):
    # Only relevant at the detail page, e.g. for a language switch menu.
    prefetch_translations = appsettings.FLUENT_FAQ_PREFETCH_TRANSLATIONS

    def get_queryset(self):
        # Not filtering active_languages(), let TranslatableSlugMixin handle everything.
        qs = super(BaseFaqDetailView, self).get_queryset()
        if self.prefetch_translations:
            qs = qs.prefetch_related('translations')
        return qs

    def get_language_choices(self):
        return appsettings.FLUENT_FAQ_LANGUAGES.get_active_choices()


class FaqCategoryDetail(BaseFaqDetailView):
    """
    Detail view for FAQ categories.
    """
    model = FaqCategory
    view_url_name = 'faqcategory_detail'


class FaqQuestionDetail(BaseFaqDetailView):
    """
    Detail view for FAQ questions.
    """
    model = FaqQuestion
    view_url_name = 'faqquestion_detail'

    def render_to_response(self, context, **response_kwargs):
        """
        Make sure the view opens at the canonical URL, or redirect otherwise.
        """
        # Redirect to the canonical URL when the category slug is incorrect.
        if self.kwargs['cat_slug'] != self.object.category.slug:
            # NOTE: this doesn't deal with multiple root nodes yet.
            return HttpResponsePermanentRedirect(self.object.get_absolute_url())

        return super(FaqQuestionDetail, self).render_to_response(context, **response_kwargs)



from django.template import Library
from fluent_faq.admin import FaqQuestionAdmin

register = Library()


@register.simple_tag()
def actions_column(faqquestion):
    return FaqQuestionAdmin.get_actions_column(faqquestion)



from django.contrib import admin
from fluent_faq.models import FaqCategory, FaqQuestion
from .faqcategoryadmin import FaqCategoryAdmin
from .faqquestionadmin import FaqQuestionAdmin

admin.site.register(FaqCategory, FaqCategoryAdmin)
admin.site.register(FaqQuestion, FaqQuestionAdmin)



"""
The manager class for the FAQ models
"""
from django.conf import settings
from fluent_faq import appsettings
from parler.managers import TranslatableManager, TranslatableQuerySet

__all__ = (
    'FaqQuestionManager',
    'FaqQuestionQuerySet',
)


class FaqBaseModelQuerySet(TranslatableQuerySet):
    """
    The QuerySet for FAQ models.
    """

    def parent_site(self, site):
        """
        Filter to the given site.
        """
        return self.filter(parent_site=site)

    def published(self):
        """
        Return only published entries for the current site.
        """
        if appsettings.FLUENT_FAQ_FILTER_SITE_ID:
            qs = self.parent_site(settings.SITE_ID)
        else:
            qs = self

        return qs

    def active_translations(self, language_code=None, **translated_fields):
        # overwritten to honor our settings instead of the django-parler defaults
        language_codes = appsettings.FLUENT_FAQ_LANGUAGES.get_active_choices(language_code)
        return self.translated(*language_codes, **translated_fields)


class FaqBaseModelManager(TranslatableManager):
    """
    Shared base logic for all FAQ models.
    """
    queryset_class = FaqBaseModelQuerySet

    def parent_site(self, site):
        """
        Filter to the given site.
        """
        return self.all().parent_site(site)

    def published(self):
        """
        Return only published entries for the current site.
        """
        return self.all().published()


# Reserve the class names for extension later
class FaqCategoryQuerySet(FaqBaseModelQuerySet):
    pass


class FaqCategoryManager(FaqBaseModelManager):
    """
    Extra methods attached to ``FaqCategory.objects`` .
    """
    queryset_class = FaqCategoryQuerySet


class FaqQuestionQuerySet(FaqBaseModelQuerySet):
    pass


class FaqQuestionManager(FaqBaseModelManager):
    """
    Extra methods attached to ``FaqQuestion.objects`` .
    """
    queryset_class = FaqQuestionQuerySet
